@@ -5,7 +5,7 @@
    System - Abstraction for running on different systems.
 
    Basic general structure..
-   
+
    ##################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
@@ -24,14 +24,14 @@
 #include <algorithm>
 #include <sstream>
 
+#include <cctype>
+#include <cerrno>
+#include <cstdlib>
+#include <cstring>
 #include <string>
 #include <vector>
-#include <ctype.h>
 #include <dirent.h>
-#include <errno.h>
 #include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -53,7 +53,7 @@ public:
    int FrontendLockFD;
    int LockFD;
    unsigned LockCount;
-   
+
    debStatusIndex *StatusFile;
 };
 
@@ -146,7 +146,7 @@ bool debSystem::Lock(OpProgress *const Progress)
       close(d->FrontendLockFD);
       return false;
    }
-   
+
    // See if we need to abort with a dirty journal
    if (CheckUpdates() == true)
    {
@@ -166,7 +166,7 @@ bool debSystem::Lock(OpProgress *const Progress)
    }
 
 	 d->LockCount++;
-      
+
    return true;
 }
 
@@ -193,7 +193,7 @@ bool debSystem::UnLock(bool NoErrors)
 {
    if (d->LockCount == 0 && NoErrors == true)
       return false;
-   
+
    if (d->LockCount < 1)
       return _error->Error(_("Not locked"));
    if (--d->LockCount == 0)
@@ -202,7 +202,7 @@ bool debSystem::UnLock(bool NoErrors)
       close(d->FrontendLockFD);
       d->LockCount = 0;
    }
-   
+
    return true;
 }
 bool debSystem::UnLockInner(bool NoErrors) {
@@ -222,7 +222,7 @@ bool debSystem::IsLocked()
 									/*}}}*/
 // System::CheckUpdates - Check if the updates dir is dirty		/*{{{*/
 // ---------------------------------------------------------------------
-/* This does a check of the updates directory (dpkg journal) to see if it has 
+/* This does a check of the updates directory (dpkg journal) to see if it has
    any entries in it. */
 bool debSystem::CheckUpdates()
 {
@@ -231,8 +231,8 @@ bool debSystem::CheckUpdates()
    DIR *DirP = opendir(File.c_str());
    if (DirP == 0)
       return false;
-   
-   /* We ignore any files that are not all digits, this skips .,.. and 
+
+   /* We ignore any files that are not all digits, this skips .,.. and
       some tmp files dpkg will leave behind.. */
    bool Damaged = false;
    for (struct dirent *Ent = readdir(DirP); Ent != 0; Ent = readdir(DirP))
@@ -373,7 +373,7 @@ bool debSystem::FindIndex(pkgCache::PkgFileIterator File,
       Found = d->StatusFile;
       return true;
    }
-   
+
    return false;
 }
 									/*}}}*/
@@ -392,7 +392,16 @@ std::string debSystem::StripDpkgChrootDirectory(std::string const &File)/*{{{*/
 									/*}}}*/
 std::string debSystem::GetDpkgExecutable()				/*{{{*/
 {
-   return StripDpkgChrootDirectory(_config->Find("Dir::Bin::dpkg","dpkg"));
+   std::string dpkg_executable = _config->Find("Dir::Bin::dpkg","dpkg");
+   if (APT::String::Startswith(dpkg_executable, "/")) {
+      struct stat buf;
+      if (stat(dpkg_executable.c_str(), &buf) != 0) {
+         _error->Warning(_("The dpkg executable set in Dir::Bin::dpkg is "
+                          "missing, falling back to using default dpkg."));
+        dpkg_executable = "dpkg";
+      }
+   }
+   return StripDpkgChrootDirectory(dpkg_executable);
 }
 									/*}}}*/
 std::vector<std::string> debSystem::GetDpkgBaseCommand()		/*{{{*/
@@ -411,6 +420,13 @@ std::vector<std::string> debSystem::GetDpkgBaseCommand()		/*{{{*/
 	 Args.push_back(Opts->Value);
       }
    }
+   auto status = _config->FindFile("Dir::State::status");
+   auto admindir = StripDpkgChrootDirectory(flNotFile(status));
+   if (admindir != "/var/lib/dpkg/")
+   {
+      Args.push_back("--admindir");
+      Args.push_back(admindir);
+   }
    return Args;
 }
 									/*}}}*/
@@ -528,10 +544,10 @@ std::vector<std::string> debSystem::ArchitecturesSupported() const		/*{{{*/
    if (dpkgMultiArch == -1)
       return archs;
 
-   FILE *dpkg = fdopen(outputFd, "r");
-   if(dpkg != NULL) {
+   if(FILE *dpkg = fdopen(outputFd, "r")) {
       char* buf = NULL;
       size_t bufsize = 0;
+      DEFER([&] { fclose(dpkg); free(buf); });
       while (getline(&buf, &bufsize, dpkg) != -1)
       {
 	 char* tok_saveptr;
@@ -548,9 +564,9 @@ std::vector<std::string> debSystem::ArchitecturesSupported() const		/*{{{*/
 	    arch = strtok_r(NULL, " ", &tok_saveptr);
 	 }
       }
-      free(buf);
-      fclose(dpkg);
    }
+   else
+        close(outputFd);
    ExecWait(dpkgMultiArch, "dpkg --print-foreign-architectures", true);
    return archs;
 }
