@@ -423,7 +423,7 @@ bool DoSource(CommandLine &CmdL)
 	 queued.insert(Last->Index().ArchiveURI(I->Path));
 
 	 // check if we have a file with that md5 sum already localy
-	 std::string localFile = flNotDir(I->Path);
+	 std::string localFile{flNotDir(I->Path)};
 	 if (FileExists(localFile) == true)
 	    if(I->Hashes.VerifyFile(localFile) == true)
 	    {
@@ -864,7 +864,7 @@ bool DoBuildDep(CommandLine &CmdL)
       return false;
 
    {
-      if (_config->FindB(AreDoingSatisfy ? "APT::Get::Satisfy-Automatic" : "APT::Get::Build-Dep-Automatic", false) == false)
+      if (_config->FindB(AreDoingSatisfy ? "APT::Get::Satisfy-Automatic" : "APT::Get::Build-Dep-Automatic", false) == false && _config->FindB("APT::Get::Mark-Auto", false) == false)
       {
 	 for (auto const &pkg: removeAgain)
 	 {
